using System.Text;
using System.Text.Json;
using Microsoft.AspNetCore.Authentication;
using Microsoft.IdentityModel.Protocols.OpenIdConnect;

namespace ReactApp.Server.Endpoints;

public static class AuthenticationEndpoint
{
    public static WebApplication MapAuthenticationEndpoints(this WebApplication app)
    {
        RouteGroupBuilder routeGroupBuilder = app.MapGroup("/bff");
        routeGroupBuilder.MapGet("/login", Login).AllowAnonymous();
        routeGroupBuilder.MapGet("/logout", Logout).AllowAnonymous();
        routeGroupBuilder.MapGet("/user", User).AllowAnonymous();

        return app;
    }

    private static async Task Login(HttpContext httpContext)
    {
        string? returnUrl = httpContext.Request.Query["returnUrl"].FirstOrDefault();

        if (string.IsNullOrEmpty(returnUrl))
        {
            returnUrl = "/";
        }

        AuthenticationProperties props = new()
        {
            RedirectUri = returnUrl
        };
        
        await httpContext.ChallengeAsync(props);
    }
    
    private static async Task Logout(HttpContext httpContext, IAuthenticationSchemeProvider authenticationAuthenticationSchemeProviderProvider)
    {
        var returnUrl = httpContext.Request.Query["returnUrl"].FirstOrDefault();
        // TODO Validate the return Url
        
        // Get rid of local cookie first
        var signInScheme = await authenticationAuthenticationSchemeProviderProvider.GetDefaultSignInSchemeAsync();
        await httpContext.SignOutAsync(signInScheme?.Name);

        if (string.IsNullOrWhiteSpace(returnUrl))
        {
            if (httpContext.Request.PathBase.HasValue)
            {
                returnUrl = httpContext.Request.PathBase;
            }
            else
            {
                returnUrl = "/";
            }
        }
        
        var props = new AuthenticationProperties
        {
            RedirectUri = returnUrl
        };    
        
        // Trigger idp logout
        await httpContext.SignOutAsync(props);
    }

    private static async Task User(HttpContext httpContext)
    {
        AuthenticateResult result = await httpContext.AuthenticateAsync();

        if (!result.Succeeded)
        {
            httpContext.Response.StatusCode = 401;
        }
        else
        {
            List<ClaimRecord> claims = [];
            claims.AddRange(GetUserClaims(result));
            claims.AddRange(GetManagementClaims(httpContext, result));

            var json = JsonSerializer.Serialize(claims);
            httpContext.Response.StatusCode = 200;
            httpContext.Response.ContentType = "application/json";
            await httpContext.Response.WriteAsync(json, Encoding.UTF8);            
        }
    }
    
    private static IEnumerable<ClaimRecord> GetUserClaims(AuthenticateResult authenticateResult)
    {
        return authenticateResult.Principal?.Claims.Select(x => new ClaimRecord(x.Type, x.Value)) ?? Enumerable.Empty<ClaimRecord>();
    }
    
    private static IEnumerable<ClaimRecord> GetManagementClaims(HttpContext context, AuthenticateResult authenticateResult)
    {
        var claims = new List<ClaimRecord>();

        var pathBase = context.Request.PathBase;

        var sessionId = authenticateResult.Principal?.FindFirst("sid")?.Value;
        if (!string.IsNullOrWhiteSpace(sessionId))
        {
            claims.Add(new ClaimRecord("logout_url", $"/bff/logout?sid={sessionId}"));
        }

        if (authenticateResult.Properties != null)
        {
            if (authenticateResult.Properties.ExpiresUtc.HasValue)
            {
                var expiresInSeconds = authenticateResult.Properties.ExpiresUtc.Value.Subtract(DateTimeOffset.UtcNow).TotalSeconds;
                claims.Add(new ClaimRecord("session_expires_in", Math.Round(expiresInSeconds)));
            }

            if (authenticateResult.Properties.Items.TryGetValue(OpenIdConnectSessionProperties.SessionState, out string? sessionState) && sessionState is not null)
            {
                claims.Add(new ClaimRecord("session_state", sessionState));
            }
        }

        return claims;
    }
    
    private record ClaimRecord(string Type, object Value);
}