using System.IdentityModel.Tokens;
using System.ServiceModel;
using System.ServiceModel.Federation;
using ApplicationServiceReference;
using Microsoft.AspNetCore.Authentication;
using Microsoft.AspNetCore.Http.HttpResults;
using Microsoft.Extensions.Options;
using ReactApp.Server.Configuration;
using ReactApp.Server.Security;
using UserServiceReference;

namespace ReactApp.Server.Endpoints;

public static class TridionDocsEndpoint
{
    public static WebApplication MapTridionDocsEndpoints(this WebApplication app)
    {
        RouteGroupBuilder routeGroupBuilder = app.MapGroup("/tridion");
        routeGroupBuilder.MapGet("/version", Version);
        routeGroupBuilder.MapGet("/whoAmI", WhoAmI);

        return app;
    }

    private static async Task<Results<Ok<string>, UnauthorizedHttpResult>> Version(
        IHttpContextAccessor contextAccessor,
        IOptionsMonitor<TridionOptions> tridionConfiguration)
    {
        string? accessToken = await contextAccessor.HttpContext!.GetTokenAsync(tokenName: "access_token");

        if (string.IsNullOrEmpty(accessToken))
        {
            return TypedResults.Unauthorized();
        }
        
        Application application = CreateApplicationChannel(accessToken, tridionConfiguration.CurrentValue.WebServices.ApplicationServiceUri);
        string version = await application.GetVersionAsync();
        return TypedResults.Ok(version);
    }
    
    private static async Task<Results<Ok<string>, UnauthorizedHttpResult>> WhoAmI(
        IHttpContextAccessor contextAccessor,
        IOptionsMonitor<TridionOptions> tridionConfiguration)
    {
        string? accessToken = await contextAccessor.HttpContext!.GetTokenAsync(tokenName: "access_token");

        if (string.IsNullOrEmpty(accessToken))
        {
            return TypedResults.Unauthorized();
        }
        
        const string xmlRequestedMetadata = @"<ishfields>
                                              <ishfield name='USERNAME' level='none'/>
                                              <ishfield name='FISHUSERDISPLAYNAME' level='none'/>
                                              <ishfield name='FISHEMAIL' level='none'/>
                                              <ishfield name='FUSERGROUP' level='none'/>
                                              <ishfield name='FISHUSERROLES' level='none' ishvaluetype='element'/>
                                              <ishfield name='FISHEXTERNALID' level='none'/>
                                            </ishfields>";        
        
        User user = CreateUserChannel(accessToken, tridionConfiguration.CurrentValue.WebServices.UserServiceUri);
        string version = await user.GetMyMetadataAsync(xmlRequestedMetadata);
        return TypedResults.Ok(version);
    }
    
    private static Application CreateApplicationChannel(string accessToken, string applicationServiceUri)
    {
        // Create the binding
        var binding = new WSFederationHttpBinding(new WSTrustTokenParameters
        {
            TokenType = "http://docs.oasis-open.org/wss/oasis-wss-saml-token-profile-1.1#SAMLV2.0",
            KeyType = SecurityKeyType.BearerKey
        });
        binding.Security.Message.EstablishSecurityContext = false;

        // Create a custom channel with a bearer token
        var factory = new ChannelFactory<Application>(binding, new EndpointAddress(applicationServiceUri));
        factory.Endpoint.EndpointBehaviors.Remove(factory.Credentials);
        var bearerCredentials = new BearerCredentials(accessToken);
        factory.Endpoint.EndpointBehaviors.Add(bearerCredentials);

        // Create the actual channel
        var channel = factory.CreateChannel();

        return channel;
    }
    
    private static User CreateUserChannel(string accessToken, string userServiceUri)
    {
        // Create the binding
        var binding = new WSFederationHttpBinding(new WSTrustTokenParameters
        {
            TokenType = "http://docs.oasis-open.org/wss/oasis-wss-saml-token-profile-1.1#SAMLV2.0",
            KeyType = SecurityKeyType.BearerKey
        });
        binding.Security.Message.EstablishSecurityContext = false;

        // Create a custom channel with a bearer token
        var factory = new ChannelFactory<User>(binding, new EndpointAddress(userServiceUri));
        factory.Endpoint.EndpointBehaviors.Remove(factory.Credentials);
        var bearerCredentials = new BearerCredentials(accessToken);
        factory.Endpoint.EndpointBehaviors.Add(bearerCredentials);

        // Create the actual channel
        var channel = factory.CreateChannel();

        return channel;
    }
}