using Microsoft.AspNetCore.Authentication.Cookies;
using Microsoft.AspNetCore.Authentication.OpenIdConnect;
using Microsoft.IdentityModel.Protocols.OpenIdConnect;
using Microsoft.IdentityModel.Tokens;
using ReactApp.Server.Configuration;
using ReactApp.Server.Endpoints;
using ReactApp.Server.Security;

var builder = WebApplication.CreateBuilder(args);
{
    builder.Services.AddHttpContextAccessor();
    builder.Services.Configure<TridionOptions>(builder.Configuration.GetSection(TridionOptions.ConfigurationName));


    // Get the authority, client id, and cookie expire timespan from the configuration
    TridionOptions tridionOptions = builder.Configuration
        .GetSection(TridionOptions.ConfigurationName).Get<TridionOptions>()!;

    builder.Services.AddAuthentication(options =>
        {
            options.DefaultScheme = CookieAuthenticationDefaults.AuthenticationScheme;
            options.DefaultChallengeScheme = OpenIdConnectDefaults.AuthenticationScheme;
            options.DefaultSignOutScheme = OpenIdConnectDefaults.AuthenticationScheme;
        })
        .AddCookie(CookieAuthenticationDefaults.AuthenticationScheme, options =>
        {
            options.ExpireTimeSpan = tridionOptions!.Authentication.CookieExpireTimeSpan;
            options.SlidingExpiration = true;
            options.Cookie.Name = "__Host-spa";
            options.Cookie.SameSite = SameSiteMode.Strict;
            
            // Having a session store sets the session id in the cookie instead of the access token
            // This is more secure and reduces the size (session id is about 336 bytes and access token around 4k bytes)
            options.SessionStore = new MemoryCacheSessionStore(tridionOptions!.Authentication.CookieExpireTimeSpan);
        })
        .AddOpenIdConnect(OpenIdConnectDefaults.AuthenticationScheme, options =>
        {
            options.Authority = tridionOptions!.Authentication.Authority;
            options.ClientId = tridionOptions!.Authentication.ClientId;
            options.ResponseType = OpenIdConnectResponseType.Code;
            options.ResponseMode = OpenIdConnectResponseMode.Query;
            options.UsePkce = true;

            options.MapInboundClaims = false;
            options.GetClaimsFromUserInfoEndpoint = true;
            options.SaveTokens = true;
            options.MapInboundClaims = false;
            
            options.Scope.Clear();
            options.Scope.Add("openid");
            options.Scope.Add("profile");
            options.Scope.Add("offline_access");

            options.TokenValidationParameters.NameClaimType = "name";
            options.TokenValidationParameters.RoleClaimType = "role";

            options.TokenValidationParameters = new TokenValidationParameters
            {
                ValidateIssuer = true,
                ValidateAudience = true,
            };
        });

    builder.Services.AddAuthorization();
}

var app = builder.Build();
{
    app.UseDefaultFiles();
    app.UseStaticFiles();

    app.UseHttpsRedirection();

    app.UseAuthentication();
    app.UseAuthorization();

    app.MapAuthenticationEndpoints();
    app.MapTridionDocsEndpoints();

    app.MapFallbackToFile("/index.html");
}
app.Run();
