using System.IdentityModel.Selectors;
using System.IdentityModel.Tokens;
using System.Security.Claims;
using System.Security.Cryptography;
using System.Security.Cryptography.X509Certificates;
using System.Text;
using System.Xml;
using Microsoft.IdentityModel.Tokens;
using Microsoft.IdentityModel.Tokens.Saml2;
using SecurityToken = System.IdentityModel.Tokens.SecurityToken;

namespace ReactApp.Server.Security;

internal sealed class BearerCredentialsSecurityTokenProvider : SecurityTokenProvider
{
	private static readonly X509Certificate2 Certificate = CreateX509Certificate2();
	private readonly BearerCredentials _tokenCredentials;

	internal BearerCredentialsSecurityTokenProvider(BearerCredentials tokenCredentials)
	{
		_tokenCredentials = tokenCredentials;
	}

	protected override SecurityToken GetTokenCore(TimeSpan timeout)
	{
		return WrapJwt(_tokenCredentials.AccessToken);
	}

	private static X509Certificate2 CreateX509Certificate2(string certName = "Client Tools")
	{
		var ecdsa = ECDsa.Create();
		var rsa = RSA.Create();
		var req = new CertificateRequest($"cn={certName}", rsa, HashAlgorithmName.SHA512, RSASignaturePadding.Pkcs1);
		var cert = req.CreateSelfSigned(DateTimeOffset.Now, DateTimeOffset.Now.AddYears(1));

		var password = Guid.NewGuid().ToString();
		return new X509Certificate2(cert.Export(X509ContentType.Pfx, password), password);
	}

	private static GenericXmlSecurityToken WrapJwt(string jwt)
	{
		// https://leastprivilege.com/2015/07/02/give-your-wcf-security-architecture-a-makeover-with-identityserver3/
		// https://github.com/IdentityServer/IdentityServer3/issues/1107
		// https://stackoverflow.com/questions/16312907/delivering-a-jwt-securitytoken-to-a-wcf-client
		// https://github.com/IdentityServer/IdentityServer3.Samples/tree/dev/source/Clients/WcfService

		var subject = new ClaimsIdentity("saml");
		subject.AddClaim(new Claim("jwt", jwt));

		// BW: Sign using the appropriate algorithm
		var securityKey = new X509SecurityKey(Certificate);
		const string algorithm = "http://www.w3.org/2001/04/xmldsig-more#rsa-sha256";
		const string digest = "http://www.w3.org/2001/04/xmlenc#sha256";
		var signingCredentials = new SigningCredentials(securityKey, algorithm, digest);

		var descriptor = new SecurityTokenDescriptor
		{
			TokenType = "http://docs.oasis-open.org/wss/oasis-wss-saml-token-profile-1.1#SAMLV2.0",
			Issuer = "urn:wrappedjwt", // BW: Changed TokenIssuerName to Issuer
			Subject = subject,
			SigningCredentials = signingCredentials
		};

		var handler = new Saml2SecurityTokenHandler
		{
			// BW: Do not include the token conditions
			SetDefaultTimesOnTokenCreation = false
		};
		var token = handler.CreateToken(descriptor) as Saml2SecurityToken;

		var sb = new StringBuilder();
		using var xmlWriter = XmlWriter.Create(sb);
		handler.WriteToken(xmlWriter, token);

		var xmlDocument = new XmlDocument();
		xmlDocument.LoadXml(sb.ToString());
		var xmlToken = new GenericXmlSecurityToken(
			xmlDocument.DocumentElement,
			null,
			DateTime.Now,
			DateTime.Now.AddHours(1),
			null,
			null,
			null);

		return xmlToken;
	}
    
}